# available_functions.py

from src.tsbuddy_version import update_package_safe as update_package
from src.tsbuddy_version import choice_form as upgrade_downgrade_choice
from src.tsbuddy import main as tsbuddy_main
from src.extracttar.extract_all import main as extract_all_main
from src.aosdl.aosdl import main as aosdl_main, lookup_ga_build, aosup
from src.logparser import main as logparser_main
from src.get_techsupport import main as get_techsupport_main
#from src.analyze.graph_hmon import main as graph_hmon_main

def get_weather(location="a specified location"):
    """Simulates getting the weather for a location."""
    print(f"🌦️  Fetching weather for {location}...")

def set_timer(duration_seconds=60, message="your reminder"):
    """Simulates setting a timer."""
    print(f"⏲️  Setting a timer for {duration_seconds} seconds with message: '{message}'")

def send_email(recipient="test@example.com", subject="Hello", body="This is a test."):
    """Simulates sending an email."""
    print(f"📧  Sending email to {recipient} with subject '{subject}'...")

def lookup_stock_price(ticker_symbol="GOOGL"):
    """Simulates looking up a stock price."""
    print(f"📈  Looking up stock price for {ticker_symbol}...")

def unknown_request():
    """Handles requests that don't match any other function."""
    print("🤔  Sorry, I'm not sure how to handle that request.")

# def aos_downloader(folder_name=None, reload_when_finished=False, found_ga_build=None):
#     aosdl_main(folder_name=folder_name, reload_when_finished=reload_when_finished, found_ga_build=found_ga_build)
#     """
#     print("\nNote: you can lookup the GA build with aosdl-ga CLI command.\n")
#     if not found_ga_build:
#         aos_major, aos_build, aos_release = get_aos_version_simple()
#     else:
#         parts = found_ga_build.split('.')
#         aos_major, aos_build, aos_release = [".".join(parts[:2]), parts[2], parts[3]]
#     print(f"Using AOS Version: {aos_major}.{aos_build}.{aos_release}")
#     """
#     print(f"Running AOS Downloader...")

# This dictionary maps the string name of the function to the actual function object.
# It's essential for the execution script.
FUNCTION_MAPPING = {
    # Simulated/demo functions
    "get_weather": get_weather,
    "set_timer": set_timer,
    "send_email": send_email,
    "lookup_stock_price": lookup_stock_price,
    "unknown_request": unknown_request,
    # Real tsbuddy functions
    "lookup_ga_build": lookup_ga_build,
    "get_techsupport_main": get_techsupport_main,
    "extract_all_main": extract_all_main,
    "tsbuddy_main": tsbuddy_main,
    "logparser_main": logparser_main,
    "aosup": aosup,
    "aos_downloader": aosdl_main,
    #"graph_hmon_main": graph_hmon_main,
    "upgrade_downgrade_choice": upgrade_downgrade_choice,
    "update_package": update_package,
}


print("\nLoading tsbuddy menu...First run will take extra time.\n")
import os
import sys
import time

from .utils.tsbuddy_version import main as check_version
# Ensure the tsbuddy_version check runs first
check_version()

from .utils.tsbuddy_version import update_package_safe as update_package
from .utils.tsbuddy_version import choice_form as upgrade_downgrade_choice
from .tslog2csv.tslog2csv import main as tsbuddy_main
#from .extract.extracttar import main as extracttar_main
from .extracttar.extract_ts_tar import main as extract_all_main
from .aos.aosdl import main as aosdl_main, lookup_ga_build, aosup
from .log_analyzer.logparser_v2 import main as logparser_main
from .log_analyzer.get_techsupport import main as get_techsupport_main
from .hmon.graph_cpu import main as graph_hmon_main
#from .clean_pycache import clean_pycache_and_pyc

print("\n" * 15)  # Clear screen by printing new lines

def update_tsbuddy():
    update_package("tsbuddy")

def print_help():
    help_text = """
\nHelp: Menu Option Details

1. Get GA Build & Upgrade (aosga):
   - Looks up the latest GA build for your switch and provides options for upgrading.
   - If you want a custom build, choose the AOS Upgrader (aosup) option to upgrade to a specific build.
   - If you only want to download an AOS image to /flash for later processing, use the AOS Downloader (aosdl) option.

2. Run tech support gatherer (ts-get):
   - Generates & gathers tech_support_complete.tar from your device, automating the collection process.

3. Run tech_support_complete.tar Extractor (ts-extract):
   - Extracts the contents of a tech_support_complete.tar archive, making logs and files accessible for analysis.
   - Use the legacy ts-extract-legacy command if you encounter issues with the new extractor.
   - The legacy extractor requires 7zip to be installed on your system and does not extract hmon data.

4. Run tech_support.log to CSV Converter (ts-csv):
   - Converts tech_support.log files into a CSV file for easier viewing and analysis.

5. Run Log Analyzer (ts-log):
   - Creates DB of switch log & console log files
   - Interactive menu for filtering 
   - Option to output the results to Excel (.xlsx)

6. Run AOS Upgrader (aosup):
   - Upgrades your OmniSwitch to the requested AOS build #, automating the upgrade process.

7. Run AOS Downloader (aosdl):
   - Downloads the requested AOS version to /flash for later processing.

8. Run HMON Graph Generator (ts-graph-cpu):
   - Generates graphs from HMON data using the new script graph_hmon.py. Must be in the same directory as HMON data files.

9. Change current directory:
   - Allows you to view and change the current working directory. Lists available directories and files, and lets you navigate to a new directory for file operations.

10. Print Help (help):
   - Shows this help text describing each menu option in detail.

🤫 Secrets and ⚙️ Settings:
- The ~/.tsbuddy_secrets file is used to store sensitive information like API keys.
- The ~/.tsbuddy_settings file is used to store user preferences and settings.

\n
"""
    print(help_text)
    time.sleep(1)  # Pause to allow user to read

def list_directory_contents(path="."):
    """List directories and files in the given path. Returns (dirs, files)."""
    dirs = []
    files = []
    for item in os.listdir(path):
        if os.path.isdir(os.path.join(path, item)):
            dirs.append(item)
        elif os.path.isfile(os.path.join(path, item)):
            files.append(item)
    print("Directories:")
    for i in dirs:
        print(f"- {i}/")
    print("Files:")
    for i in files:
        print(f"- {i}")
    return dirs, files

def change_directory():
    # Print current directory
    print(f"Current directory: {os.getcwd()}\n")
    list_directory_contents()
    # Prompt user for new directory
    print("\nEnter the path to the new directory (or press Enter to keep current):")
    print("You can also use relative paths like '../' to go up a directory.")
    new_dir = input("Enter the path to the new directory: ").strip()
    if os.path.isdir(new_dir):
        os.chdir(new_dir)
        print(f"Directory changed to: {os.getcwd()}")
        list_directory_contents()
    else:
        print("Current directory remains unchanged: ", os.getcwd())

def menu():
    menu_options = [
        {" Get GA Build, Family, or Upgrade (aosga)": lookup_ga_build},
        {" Run tech support gatherer (ts-get)": get_techsupport_main},
        {" Run tech_support_complete.tar Extractor (ts-extract) (ts-extract-legacy)": extract_all_main},
        {" Run tech_support.log to CSV Converter (ts-csv)": tsbuddy_main},
        {" Run Log Analyzer (ts-log)": logparser_main},
        {" Run AOS Upgrader (aosup)": aosup},
        {" Run AOS Downloader (aosdl)": aosdl_main},
        {" Run CPU Graph (ts-graph-cpu)": graph_hmon_main},
        {" Change current directory or list contents (cd)": change_directory},
        # {"Clear pycache and .pyc files (ts-clean)": clean_pycache_and_pyc},
        {"Upgrade or downgrade tsbuddy": upgrade_downgrade_choice},
        {"Show help info": print_help},
    ]
    #print("\n       (•‿•)  Hey there, buddy!")
    #print(ale_ascii)
    try:
        print("\n   ( ^_^)ノ  Hey there, tsbuddy is at your service!")
    except:
        print("\n   ( ^_^)/  Hey there, tsbuddy is at your service!")
    print("\n Skip this menu by running the CLI commands directly (provided in parentheses below), e.g. `ts-get`.\n")
    while True:
        try:
            print("\n=== 🛎️  ===")
        except:
            print("\n=== Menu ===")
        for idx, opt in enumerate(menu_options, 1):
            print(f"{idx}. {list(opt.keys())[0]}")
        try:
            print("\n0. Exit  (つ﹏<) \n")
        except:
            print("\n0. Exit  (T_T) \n")
        choice = input("Select an option: ").strip()
        if choice.isdigit() and 1 <= int(choice) <= len(menu_options):
            try:
                #print(f"\n   ( ^_^)ノ⌒☆   \n")
                print(f"\n   ( ^_^)ノ🛎️   \n")
            except:
                #print(f"\n   ( ^_^)/🕭   \n")
                pass
            # Get the function from the selected option
            selected_func = list(menu_options[int(choice)-1].values())[0]
            try:
                selected_func()
            except Exception as e:
                print(f"\nError: {e}\nReturning to menu...\n")
        elif choice.lower() == 'cd' or choice.lower() == 'ls':
            change_directory()
        elif choice == '0':
            print("Exiting...\n\n  (x_x) \n")
            sys.exit(0)
        else:
            print("\nInvalid choice. Please try again.")

if __name__ == "__main__":
    menu()


import re
import csv
import os
from typing import List, Dict
from datetime import datetime

# Column definitions
COLUMN_DEFINITIONS = {
    "show hardware-info": [
        "Chassis",
        "CPU Manufacturer",
        "CPU Model",
        "Flash Manufacturer",
        "Flash size",
        "RAM size",
        "FPGA version",
        "U-Boot Version",
        "Power Supplies Present",
        "NIs Present",
    ],
    "show chassis": [
        "Chassis ID",
        "Role",
        "Model Name",
        "Module Type",
        "Description",
        "Part Number",
        "Hardware Revision",
        "Serial Number",
        "Manufacture Date",
        "Admin Status",
        "Operational Status",
        "Number Of Resets",
        "MAC Address",
    ],
    "show module long": [
        "Module ID",
        "Slot",
        "Chassis",
        "Model Name",
        "Module Type",
        "Description",
        "Part Number",
        "Hardware Revision",
        "Serial Number",
        "Manufacture Date",
        "FPGA - Physical 1",
        "Admin Status",
        "Operational Status",
        "Max Power",
        "CPU Model Type",
        "MAC Address",
        "UBOOT Version",
        "POE-Software Version",
        "POE-Hardware Version",
    ],
    "show transceivers": [
        "Chassis ID",
        "Slot",
        "Port Number",
        "Manufacturer Name",
        "Part Number",
        "ALU Model Name",
        "ALU Model Number",
        "Hardware Revision",
        "Serial Number",
        "Manufacture Date",
        "Laser Wave Length",
        "Admin Status",
        "Operational Status",
    ],
    "show fan": [
        "Chassis/Tray",
        "Fan",
        "Functional",
        "Speed",
    ],
    "show powersupply": [
        "Chassis/PS",
        "Power",
        "PS Type",
        "Status",
        "Location",
    ],
    "show temperature": [
        "Chassis/Device",
        "Current",
        "Range",
        "Danger",
        "Thresh",
        "Status",
    ],
    "show system": [
        "Description",
        "Object ID",
        "Up Time",
        "Contact",
        "Name",
        "Location",
        "Services",
        "Date & Time",
        "Primary CMM - Available (bytes)",
        "Primary CMM - Comments",
        "Secondary CMM - Available (bytes)",
        "Secondary CMM - Comments",
    ],
    "show running-directory": [
        "Running CMM",
        "CMM Mode",
        "Current CMM Slot",
        "Running Directory",
        "Certify/Restore Status",
        "Flash Between CMMs",
        "Running Configuration",
    ],
    "show microcode": [
        "Location",
        "Directory",
        "Package",
        "Release",
        "Size",
        "Description",
    ],
    "show license-info": [
        "VC",
        "Device",
        "License",
        "Type",
        "Time Remaining (Days)",
        "Upgrade Status",
        "Expiration Date",
    ],
    "show lldp remote-system": [
        "Local Port",
        "Chassis IP",
        "Port",
        "Remote ID",
        "Chassis Subtype",
        "Port Subtype",
        "Port Description",
        "System Name",
        "System Description",
        "Capabilities Supported",
        "Capabilities Enabled",
        "Management IP Address",
        "MED Device Type",
        "MED Capabilities",
        "MED Extension TLVs Present",
        "MED Power Type",
        "MED Power Source",
        "MED Power Priority",
        "MED Power Value",
        "Remote port MAC/PHY AutoNeg",
        "Mau Type",
    ],
    "show aaa authentication": [
        "Service Type",
        "Authentication",
        "1st Authentication Server",
        "2nd Authentication Server",
        "3rd Authentication Server",
        "4th Authentication Server",
    ],
    "show health": [
        "Resource",
        "Current",
        "1 Min Avg",
        "1 Hr Avg",
        "1 Day Avg",
    ],
    "show health all cpu": [
        "CPU",
        "Current",
        "1 Min Avg",
        "1 Hr Avg",
        "1 Day Avg",
    ],
    "show vlan": [
        "VLAN",
        "Type",
        "Admin State",
        "Operational",
        "IP",
        "MTU",
        "Name",
    ],
    "show spantree": [
        "VLAN",
        "STP Status",
        "Protocol",
        "Priority",
        "Spanning Tree Path Cost Mode",
    ],
    "show spantree ports active": [
        "VLAN",
        "Port",
        "Oper Status",
        "Path Cost",
        "Role",
        "Loop Guard",
        "Note",
    ],
    "show interfaces status": [
        "Chas/Slot/Port",
        "Admin Status",
        "Auto Nego",
        "Detected Speed (Mbps)",
        "Detected Duplex",
        "Detected Pause",
        "Detected FEC",
        "Configured Speed (Mbps)",
        "Configured Duplex",
        "Configured Pause",
        "Configured FEC",
        "Link Trap",
        "EEE Status",
    ],
    "show interfaces counters": [
        "Interface",
        "InOctets",
        "OutOctets",
        "InUcastPkts",
        "OutUcastPkts",
        "InMcastPkts",
        "OutMcastPkts",
        "InBcastPkts",
        "OutBcastPkts",
        "InPauseFrames",
        "OutPauseFrames",
        "InPkts/s",
        "OutPkts/s",
        "InBits/s",
        "OutBits/s",
    ],
    "show ip interface": [
        "Name",
        "IP Address",
        "Subnet Mask",
        "Status",
        "Forward",
        "Device",
        "Flags",
    ],
    "show ip config": [
        "IP directed-broadcast",
        "IP default TTL",
        "Distributed ARP",
        "Anycast MAC",
        "Proxy-arp aging-time",
    ],
    "show ip protocols": [
        "RIP status",
        "OSPF status",
        "ISIS status",
        "BGP status",
        "PIM status",
        "DVMRP status",
        "RIPng status",
        "OSPF3 status",
        "LDP status",
        "VRRP status",
    ],
    "show ip dos statistics": [
        "port scan",
        "ping of death",
        "land",
        "loopback-src",
        "invalid-ip",
        "invalid-multicast",
        "unicast dest-ip/multicast-mac",
        "ping overload",
        "arp flood",
        "arp poison",
        "anti-spoof",
        "gratuitous-arp",
        "ip-options-filter",
    ],
    "show snmp statistics": [
        "snmpInPkts",
        "snmpOutPkts",
        "snmpInBadVersions",
        "snmpInBadCommunityNames",
        "snmpInBadCommunityUses",
        "snmpInASNParseErrs",
        "snmpEnableAuthenTraps",
        "snmpSilentDrop",
        "snmpProxyDrops",
        "snmpInTooBigs",
        "snmpInNoSuchNames",
        "snmpInBadValues",
        "snmpInReadOnlys",
        "snmpInGenErrs",
        "snmpInTotalReqVars",
        "snmpInTotalSetVars",
        "snmpInGetRequests",
        "snmpInGetNexts",
        "snmpInSetRequests",
        "snmpInGetResponses",
        "snmpInTraps",
        "snmpOutTooBigs",
        "snmpOutNoSuchNames",
        "snmpOutBadValues",
        "snmpOutGenErrs",
        "snmpOutGetRequests",
        "snmpOutGetNexts",
        "snmpOutSetRequests",
        "snmpOutGetResponses",
        "snmpOutTraps",
        "snmpUnknownSecurityModels",
        "snmpInvalidMsgs",
        "snmpUnknownPDUHandlers",
        "snmpUnavailableContexts",
        "snmpUnknownContexts",
        "usmStatsUnsupportedSecLevels",
        "usmStatsNotInTimeWindows",
        "usmStatsUnknownUserNames",
        "usmStatsUnknownEngineIDs",
        "usmStatsWrongDigests",
        "usmStatsDecryptionErrors",
        "snmpTsmInvalidCaches",
        "snmpTsmInadequateSecurityLevels",
        "snmpTsmUnknownPrefixes",
        "snmpTsmInvalidPrefixes",
        "snmpTlstmSessionOpens",
        "snmpTlstmSessionClientCloses",
        "snmpTlstmSessionOpenErrors",
        "snmpTlstmSessionAccepts",
        "snmpTlstmSessionServerCloses",
        "snmpTlstmSessionNoSessions",
        "snmpTlstmSessionInvalidClientCertificates",
        "snmpTlstmSessionUnknownServerCertificate",
        "snmpTlstmSessionInvalidServerCertificates",
        "snmpTlstmSessionInvalidCaches",
        "snmpEngineID",
        "snmpEngineBoots",
        "snmpEngineTime",
        "snmpEngineMaxMessageSize",
    ],
    "show virtual-chassis topology": [
        "Oper Chas",
        "Role",
        "Status",
        "Config Chas ID",
        "Oper Priority",
        "Group",
        "MAC-Address",
        "Local Chassis",
    ],
    "show virtual-chassis consistency": [
        "Chassis ID",
        "Config Chas ID",
        "Status",
        "Oper Type",
        "Oper Group",
        "Hello Interv",
        "Oper Control Vlan",
        "Config Control Vlan",
        "License",
    ],
    "show virtual-chassis vf-link member-port": [
        "Chassis/VFLink ID",
        "Chassis/Slot/Port",
        "Oper",
        "Is Primary",
        "VFLink Mode",
    ],
    "show virtual-chassis chassis-reset-list": [
        "Chas",
        "Chassis reset list",
    ],
    "show virtual-chassis slot-reset-list": [
        "Chas",
        "Slot",
        "Reset status",
    ],
    "show virtual-chassis vf-link": [
        "Chassis/VFLink ID",
        "Oper",
        "Primary Port",
        "Config Port",
        "Active Port",
        "Def Vlan",
        "Speed Type",
        "VFLink Mode",
    ],
    "show virtual-chassis auto-vf-link-port": [
        "Chassis/Slot/Port",
        "Chassis/VFLink ID",
        "VFLink member status",
    ],
    "show virtual-chassis neighbors": [
        "Chas ID",
        "VFL 0",
        "VFL 1",
    ],
    "debug show virtual-chassis topology": [
        "Oper Chas",
        "Role",
        "Status",
        "Config Chas ID",
        "Oper Priority",
        "Group",
        "MAC-Address",
        "System Ready",
        "Local Chassis",
    ],
    "debug show virtual-chassis status": [
        "ID",
        "Level",
        "Parameter",
        "Value",
        "Timestamp",
        "Status",
    ],
    "debug show virtual-chassis connection": [
        "Chas",
        "MAC-Address",
        "Local IP",
        "Remote IP",
        "Status",
    ],
    "show cloud-agent status": [
        "Admin State",
        "Activation Server State",
        "Device State",
        "Error State",
        "Cloud Group",
        "DHCP Address",
        "DHCP IP Address Mask",
        "Gateway",
        "Activation Server",
        "Network ID",
        "NTP Server",
        "DNS Server",
        "DNS Domain",
        "Proxy Server",
        "VPN Server",
        "Preprovision Server",
        "OV Tenant",
        "VPN DPD Time (sec)",
        "Image Server",
        "Image Download Retry Count",
        "Discovery Interval (min)",
        "Time to next Call Home (sec)",
        "Call Home Timer Status",
        "Discovery Retry Count",
        "Certificate Status",
        "Thin Client",
        "Retry Call-Home time remaining",
    ],
    "show pkgmgr": [
        "Name",
        "Version",
        "Status",
        "Install Script",
    ],
    "show appmgr": [
        "Application",
        "Status",
        "Package Name",
        "User",
        "Status Time Stamp",
    ],
    "show naas license": [
        "Chas ID",
        "Serial Number",
        "Device Mode",
        "Device State",
        "Call-home Period",
        "Grace Period",
        "Valid Licenses",
        "Expiry Day",
        "Expiry Time",
    ],
    "show naas-agent status": [
        "Proxy Server",
        "Activation Server",
        "DNS Server",
        "DNS Domain",
        "NTP Server",
        "Call Home Timer Status",
        "Time to next call-home (min)",
        "Grace Period (days)",
        "Grace Period (cause)",
    ],
    "debug show capability naas": [
        "Chas ID",
        "Naas",
        "Decided",
        "Esntial",
        "Advanced",
        "DC",
        "10G",
        "MACSEC",
        "MPLS",
        "GRACE",
        "DEGR",
        "MGMT",
        "UPGRADE",
        "GRC_MGMT",
        "GRCUPGRADE",
        "DEGMGMT",
        "DEGUPGRADE",
        "ADVROUTING",
    ],
    "show ntp server status": [
        "IP address",
        "Host mode",
        "Peer mode",
        "Prefer",
        "Version",
        "Key",
        "Stratum",
        "Minpoll",
        "Maxpoll",
        "Poll",
        "When",
        "Delay",
        "Offset",
        "Dispersion",
        "Root distance",
        "Precision",
        "Reference IP",
        "Status",
        "Uptime count",
        "Reachability",
        "Unreachable count",
        "Stats reset count",
        "Packets sent",
        "Packets received",
        "Duplicate packets",
        "Bogus origin",
        "Bad authentication",
        "Bad dispersion",
        "Last Event",
    ],
    "show ntp status": [
        "Current time",
        "Last NTP update",
        "Server reference",
        "Client mode",
        "Broadcast client mode",
        "Broadcast delay (microseconds)",
        "Clock status",
        "Stratum",
        "Maximum Associations Allowed",
        "Authentication",
        "Source IP Configured",
        "VRF Name",
    ],
    "show ntp keys": [
        "Key",
        "Status",
    ],
    "show capability profile": [
        "Configured Profile",
        "Active Profile",
        "Configured TCAM Mode",
        "Active TCAM mode",
    ],
}

# Parsers
def parse_hardware_info(output: str) -> List[Dict[str, str]]:
    columns = COLUMN_DEFINITIONS["show hardware-info"]
    data, current = [], {}
    for line in output.strip().splitlines():
        if match := re.match(r'^Chassis (\d+)', line):
            if current:
                data.append(current)
                current = {}
            current["Chassis"] = match.group(1)
        elif ':' in line:
            key, value = map(str.strip, line.split(':', 1))
            if key in columns:
                current[key] = value
    if current:
        data.append(current)
    return data


def parse_chassis(output: str) -> List[Dict[str, str]]:
    columns = COLUMN_DEFINITIONS["show chassis"]
    data, current = [], {}
    pattern = re.compile(r'^(Local|Remote) Chassis ID (\d+) \((\w+)\)')
    for line in output.strip().splitlines():
        match = pattern.match(line)
        if match:
            if current:
                data.append(current)
                current = {}
            _, chassis_id, role = match.groups()
            current["Chassis ID"] = chassis_id
            current["Role"] = role
        elif ':' in line:
            key, value = map(str.strip, line.split(':', 1))
            if key in columns:
                current[key] = value.strip(',')
    if current:
        data.append(current)
    return data


def parse_module_long(output: str) -> List[Dict[str, str]]:
    columns = COLUMN_DEFINITIONS["show module long"]
    data = []
    current = {}

    # Persistent context for Module ID, Chassis, Slot
    context = {
        "Module ID": None,
        "Chassis": None,
        "Slot": None
    }

    lines = output.strip().splitlines()

    module_id_pattern = re.compile(r'^Module ID (\d+)')
    slot_pattern = re.compile(r'^Module in slot ([\w-]+)')
    chassis_slot_pattern = re.compile(r'^Module in chassis (\d+) slot (\d+)')

    for line in lines:
        line = line.strip()
        if not line:
            continue

        # Match and store Module ID context
        if module_id_match := module_id_pattern.match(line):
            context["Module ID"] = module_id_match.group(1)

        # Start new record: Module in chassis X slot Y
        elif chassis_slot_match := chassis_slot_pattern.match(line):
            if current:
                data.append(current)
            context["Chassis"] = chassis_slot_match.group(1)
            context["Slot"] = chassis_slot_match.group(2)
            current = {k: v for k, v in context.items() if v is not None}

        # Start new record: Module in slot CMM-A (no chassis)
        elif slot_match := slot_pattern.match(line):
            if current:
                data.append(current)
            context["Slot"] = slot_match.group(1)
            context["Chassis"] = None  # Clear chassis if not present
            current = {k: v for k, v in context.items() if v is not None}

        # Key-value fields
        elif ':' in line:
            key, value = map(str.strip, line.split(':', 1))
            key = key.rstrip(',')
            value = value.rstrip(',')
            if key in columns:
                current[key] = value

    if current:
        data.append(current)

    return data


def parse_transceivers(output: str) -> List[Dict[str, str]]:
    columns = COLUMN_DEFINITIONS["show transceivers"]
    data = []
    current = {}
    current_chassis_id = None

    lines = output.strip().splitlines()

    chassis_pattern = re.compile(r"^Chassis ID (\d+)")
    transceiver_pattern = re.compile(r"^Slot (\d+) Transceiver (\d+)")

    for line in lines:
        line = line.strip()
        if not line:
            continue

        chassis_match = chassis_pattern.match(line)
        transceiver_match = transceiver_pattern.match(line)

        if chassis_match:
            current_chassis_id = chassis_match.group(1)

        elif transceiver_match:
            if current:
                data.append(current)
                current = {}

            slot, port_number = transceiver_match.groups()
            current["Chassis ID"] = current_chassis_id
            current["Slot"] = slot
            current["Port Number"] = port_number

        elif ':' in line:
            key, value = map(str.strip, line.split(":", 1))
            key = key.rstrip(',')
            value = value.rstrip(',')
            if key in columns:
                current[key] = value

    if current:
        data.append(current)

    return data


def parse_fan(output: str) -> List[Dict[str, str]]:
    columns = COLUMN_DEFINITIONS["show fan"]
    data = []

    lines = output.strip().splitlines()
    header_found = False

    for line in lines:
        line = line.strip()

        if not line:
            continue

        # Skip the header and separator
        if not header_found:
            if all(col in line for col in ["Chassis/Tray", "Fan", "Functional", "Speed"]):
                header_found = True  # Header line found
            continue
        elif set(line) <= set("-+| "):
            continue  # Skip the separator line

        # Split columns by whitespace assuming fixed-width format
        parts = re.split(r'\s{2,}', line)
        if len(parts) == 4:
            # Precede Chassis/Tray value with a single quote
            #parts[0] = "'" + parts[0]
            row = dict(zip(columns, parts))
            data.append(row)

    return data


def parse_powersupply(output: str) -> List[Dict[str, str]]:
    columns = COLUMN_DEFINITIONS["show powersupply"]
    data = []

    lines = output.strip().splitlines()
    header_found = False

    for line in lines:
        line = line.strip()

        if not line:
            continue

        # Skip header and separator lines
        if not header_found:
            if all(col in line for col in ["Chassis/PS", "Power", "Type", "Status", "Location"]):
                header_found = True
            continue
        elif set(line) <= set("-+ "):
            continue

        # Skip the total summary line
        if line.lower().startswith("total"):
            continue

        # Parse the data line
        parts = re.split(r'\s{2,}', line)
        if len(parts) == 5:
            row = dict(zip(columns, parts))
            data.append(row)

    return data


def parse_temperature(output: str) -> List[Dict[str, str]]:
    columns = COLUMN_DEFINITIONS["show temperature"]
    data = []

    lines = output.strip().splitlines()
    header_found = False

    for line in lines:
        line = line.strip()
        if not line:
            continue

        # Detect the header line
        if not header_found:
            if all(col in line for col in ["Chassis/Device", "Current", "Range", "Danger", "Thresh", "Status"]):
                header_found = True
            continue
        elif set(line) <= set("-+| "):
            continue  # Skip separator line

        # Split fixed-width fields by 2+ spaces
        parts = re.split(r'\s{2,}', line)
        if len(parts) == 6:
            row = dict(zip(columns, parts))

            # Ensure "Chassis/Device" is interpreted as text in Excel
            #if "/" in row["Chassis/Device"]:
            #    row["Chassis/Device"] = f"=\"{row['Chassis/Device']}\""

            data.append(row)

    return data


def parse_system(output: str) -> List[Dict[str, str]]:
    columns = COLUMN_DEFINITIONS["show system"]
    data = {}

    # General system fields
    general_pattern = re.compile(r"^(.*?):\s+(.*?),?$")
    for line in output.splitlines():
        line = line.strip()
        if not line or line.startswith("Flash Space:"):
            continue
        match = general_pattern.match(line)
        if match:
            key, value = match.groups()
            key = key.strip()
            value = value.strip()
            if key in columns:
                data[key] = value

    # Match Primary CMM block
    primary_match = re.search(r"Primary CMM:\s*(.*?)\s*(?:Secondary CMM:|$)", output, re.DOTALL)
    if primary_match:
        primary_block = primary_match.group(1)
        for line in primary_block.splitlines():
            if ':' in line:
                key, value = map(str.strip, line.split(":", 1))
                full_key = f"Primary CMM - {key}"
                if full_key in columns:
                    data[full_key] = value.strip(',')

    # Match Secondary CMM block
    secondary_match = re.search(r"Secondary CMM:\s*(.*)", output, re.DOTALL)
    if secondary_match:
        secondary_block = secondary_match.group(1)
        for line in secondary_block.splitlines():
            if ':' in line:
                key, value = map(str.strip, line.split(":", 1))
                full_key = f"Secondary CMM - {key}"
                if full_key in columns:
                    data[full_key] = value.strip(',')

    return [data] if data else []


def parse_running_directory(output: str) -> List[Dict[str, str]]:
    columns = COLUMN_DEFINITIONS["show running-directory"]
    data = {key: "" for key in columns}

    current_block = None
    for line in output.splitlines():
        line = line.strip()
        if not line:
            continue

        if line == "CONFIGURATION STATUS":
            current_block = "CONFIGURATION STATUS"
            continue
        elif line == "SYNCHRONIZATION STATUS":
            current_block = "SYNCHRONIZATION STATUS"
            continue

        if ':' in line:
            key, value = map(str.strip, line.split(":", 1))
            value = value.rstrip(',')

            if current_block == "CONFIGURATION STATUS":
                if key == "Running configuration":
                    data["Running Directory"] = value
                elif key in columns:
                    data[key] = value
            elif current_block == "SYNCHRONIZATION STATUS":
                if key == "Running Configuration":
                    data["Running Configuration"] = value
                elif key in columns:
                    data[key] = value

    return [data]


def parse_microcode(output: str, location: str) -> List[Dict[str, str]]:
    columns = COLUMN_DEFINITIONS["show microcode"]
    data = []

    lines = output.strip().splitlines()

    if not lines:
        return data

    # Step 1: Find the directory line
    directory = ""
    for line in lines:
        line = line.strip()
        if line:
            directory = line
            break

    # Step 2: Parse actual data lines after separator
    parsing = False
    for line in lines:
        line = line.strip()

        if not line:
            continue

        if re.match(r"^-+\+.*", line):
            parsing = True
            continue

        if not parsing:
            continue

        # Regex pattern to extract: Package, Release, Size, Description
        match = re.match(r'^(\S+)\s+(\S+)\s+(\d+)\s+(.+)$', line)
        if match:
            package, release, size, description = match.groups()
            row = {
                "Location": location,
                "Directory": f"=\"{directory}\"",  # Protect from Excel path parsing
                "Package": package,
                "Release": release,
                "Size": size,
                "Description": description
            }
            data.append(row)

    return data


def parse_license_info(output: str) -> List[Dict[str, str]]:
    columns = COLUMN_DEFINITIONS["show license-info"]
    data = []
    
    lines = output.strip().splitlines()
    header_found = False

    for line in lines:
        line = line.strip()

        if not line:
            continue

        # Skip the header and separator
        if not header_found:
            if all(col in line for col in ["VC", "device", "License", "Type", "Remaining", "Status", "Date"]):
                header_found = True  # Header line found
            continue
        elif set(line) <= set("-+| "):
            continue  # Skip the separator line

        # Split columns by whitespace (accounting for multiple spaces between columns)
        parts = re.split(r'\s{2,}', line)
        if len(parts) == 7:
            row = dict(zip(columns, parts))
            data.append(row)

    return data


def parse_lldp_remote_system(raw_text: str) -> List[Dict[str, str]]:
    """
    Parses the show lldp remote-system section.
    """
    entries = []
    entry_pattern = re.compile(
        r"Remote LLDP nearest-bridge Agents on Local Port (\S+):\s*"
        r"Chassis\s+([\w\.:]+),\s+Port\s+([\S]+):\s*"
        r"(.*?)\s*(?=(Remote LLDP nearest-bridge|$))",
        re.DOTALL
    )
    
    matches = entry_pattern.findall(raw_text)
    
    for match in matches:
        local_port = match[0]
        chassis_ip = match[1]
        port = match[2]
        details = match[3]
        
        # Extracting key-value pairs
        detail_pattern = re.compile(r"^\s*([\w\s]+)\s*=\s*([\w\.\:,\-\/\(\)]+)?", re.MULTILINE)
        detail_matches = detail_pattern.findall(details)
        
        data = {
            "Local Port": local_port,
            "Chassis IP": chassis_ip,
            "Port": port,
        }
        
        # Populating the dictionary with available data
        for key, value in detail_matches:
            # Clean up any leading or trailing whitespace
            key = key.strip()
            value = value.strip().rstrip(",.") if value else None
            data[key] = value
        
        entries.append(data)
    
    return entries
    

def parse_aaa_authentication(raw_text: str) -> List[Dict[str, str]]:
    """
    Parses the 'show aaa authentication' section with support for optional 'Authentication = Use Default' entries.
    """
    entries = []
    current_entry = {}
    lines = raw_text.strip().splitlines()

    for line in lines:
        line = line.strip()

        if line.startswith("Service type ="):
            # Save the previous entry if it exists
            if current_entry:
                entries.append(current_entry)
                current_entry = {}
            service_type = line.split("=", 1)[1].strip()
            current_entry["Service Type"] = service_type

        elif line.startswith("Authentication ="):
            value = line.split("=", 1)[1].strip().rstrip(',')
            current_entry["Authentication"] = value

        elif "authentication server" in line:
            match = re.match(r"(\d+)(?:st|nd|rd|th) authentication server\s*=\s*(.+?),?$", line)
            if match:
                index = int(match.group(1))
                server = match.group(2).strip()
                suffix = (
                    f"{index}st" if index == 1 else
                    f"{index}nd" if index == 2 else
                    f"{index}rd" if index == 3 else
                    f"{index}th"
                )
                key = f"{suffix} Authentication Server"
                current_entry[key] = server

    # Append the final entry
    if current_entry:
        entries.append(current_entry)

    return entries


def parse_health(raw_text: str) -> List[Dict[str, str]]:
    """
    Parses the 'show health' section of the log.
    """
    rows = []
    lines = raw_text.strip().splitlines()

    # Skip headers, look for lines with actual data (usually below the separator line)
    data_started = False
    for line in lines:
        if re.match(r"^-+\+.*-+$", line):  # separator line with dashes and pluses
            data_started = True
            continue
        if not data_started:
            continue

        parts = re.split(r"\s{2,}", line.strip())
        if len(parts) == 5:
            rows.append({
                "Resource": parts[0],
                "Current": parts[1],
                "1 Min Avg": parts[2],
                "1 Hr Avg": parts[3],
                "1 Day Avg": parts[4]
            })

    return rows


def parse_health_all_cpu(raw_text: str) -> List[Dict[str, str]]:
    """
    Parses the 'show health all cpu' section.
    """
    entries = []
    lines = raw_text.strip().splitlines()

    for line in lines:
        # Match lines that look like: "Slot  1/1    22    19    19    19"
        match = re.match(r'^(Slot\s+\S+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)', line)
        if match:
            entries.append({
                "CPU": match.group(1),
                "Current": match.group(2),
                "1 Min Avg": match.group(3),
                "1 Hr Avg": match.group(4),
                "1 Day Avg": match.group(5)
            })

    return entries


def parse_vlan(raw_text: str) -> List[Dict[str, str]]:
    """
    Parses the 'show vlan' section into a list of dictionaries.
    Handles space in the 'name' field.
    """
    entries = []
    lines = raw_text.strip().splitlines()
    data_started = False

    for line in lines:
        if re.match(r"-+\+-+", line):  # separator line
            data_started = True
            continue
        if not data_started or not line.strip():
            continue

        # Use regex to extract the first 6 fields; the rest is the name
        match = re.match(
            r'(\d+)\s+(\w+)\s+(\w+)\s+(\w+)\s+(\w+)\s+(\d+)\s+(.*)', line.strip()
        )
        if match:
            entry = {
                "VLAN": match.group(1),
                "Type": match.group(2),
                "Admin State": match.group(3),
                "Operational": match.group(4),
                "IP": match.group(5),
                "MTU": match.group(6),
                "Name": match.group(7).strip()
            }
            entries.append(entry)

    return entries


def parse_spantree(output: str) -> List[Dict[str, str]]:
    columns = COLUMN_DEFINITIONS["show spantree"]
    data = []
    spanning_tree_mode = ""

    lines = output.strip().splitlines()

    # Extract global config like Spanning Tree Path Cost Mode
    for idx, line in enumerate(lines):
        if "Spanning Tree Path Cost Mode" in line:
            match = re.search(r"Spanning Tree Path Cost Mode\s*:\s*(\S+)", line)
            if match:
                spanning_tree_mode = match.group(1)
            lines = lines[idx + 1:]  # Skip to table
            break

    header_found = False
    for line in lines:
        line = line.strip()
        if not line or set(line) <= set("-+ "):
            continue

        if not header_found:
            if all(keyword in line for keyword in ["Vlan", "STP", "Protocol", "Priority"]):
                header_found = True
            continue

        # Parse table rows
        parts = re.split(r'\s{2,}', line)
        if len(parts) == 4:
            row = {
                "VLAN": parts[0],
                "STP Status": parts[1],
                "Protocol": parts[2],
                "Priority": parts[3],
                "Spanning Tree Path Cost Mode": spanning_tree_mode
            }
            data.append(row)
    return data


def parse_spantree_ports_active(output: str) -> List[Dict[str, str]]:
    columns = COLUMN_DEFINITIONS["show spantree ports active"]
    data = []

    lines = output.strip().splitlines()
    header_found = False

    for line in lines:
        line = line.strip()
        if not line or set(line) <= set("-+ "):
            continue

        if not header_found:
            if all(keyword in line for keyword in ["Vlan", "Port", "Oper Status", "Path Cost", "Role", "Loop Guard"]):
                header_found = True
            continue

        # Split columns using 2+ spaces (assuming fixed-width formatting)
        parts = re.split(r'\s{2,}', line)
        if len(parts) >= 6:
            # Ensure 'Note' is included even if blank
            while len(parts) < 7:
                parts.append("")
            row = dict(zip(columns, parts))
            data.append(row)

    return data


def parse_interfaces_status(output):
    """
    Parses the 'show interfaces status' command output using regex-based splitting.
    Returns a list of dictionaries, one per interface.
    """
    data = []
    capture = False

    for line in output.splitlines():
        # Start capturing when data lines begin (match chassis/slot/port format)
        if re.match(r'^\s*\d+/\d+/\d+', line):
            capture = True

        if capture:
            # Skip separator lines
            if re.match(r"^\s*[-+]+\s*$", line):
                continue

            # Split line into tokens
            tokens = re.split(r'\s+', line.strip())

            # Defensive check to ensure expected number of fields
            if len(tokens) >= 13:
                entry = {
                    "Chas/Slot/Port": tokens[0],
                    "Admin Status": tokens[1],
                    "Auto Nego": tokens[2],
                    "Detected Speed (Mbps)": tokens[3],
                    "Detected Duplex": tokens[4],
                    "Detected Pause": tokens[5],
                    "Detected FEC": tokens[6],
                    "Configured Speed (Mbps)": tokens[7],
                    "Configured Duplex": tokens[8],
                    "Configured Pause": tokens[9],
                    "Configured FEC": tokens[10],
                    "Link Trap": tokens[11],
                    "EEE Status": tokens[12]
                }
                data.append(entry)

    return data


def parse_interfaces_counters(text):
    interfaces = []
    current_interface = None
    current_data = {}

    for line in text.splitlines():
        line = line.strip()
        if not line:
            continue

        # Match interface line (e.g., "3/1/47  ,")
        iface_match = re.match(r'^(\d+/\d+/\d+)\s*,', line)
        if iface_match:
            # Store previous interface data
            if current_interface and current_data:
                interfaces.append({"Interface": current_interface, **current_data})
            # Start new interface
            current_interface = iface_match.group(1)
            current_data = {}
            continue

        # Match all key=value pairs (including keys with slashes)
        for kv in re.findall(r'([\w/]+)\s*=\s*([\d]+)', line):
            key, value = kv
            current_data[key] = int(value)

    # Append last interface
    if current_interface and current_data:
        interfaces.append({"Interface": current_interface, **current_data})

    return interfaces


def parse_ip_interface(text):
    interfaces = []
    lines = text.strip().splitlines()
    header_found = False

    for line in lines:
        if re.match(r'^-+\+-+', line):
            header_found = True
            continue
        if not header_found or not line.strip():
            continue

        # Pattern for the fields
        match = re.match(
            r'^(.+?)\s+(\d{1,3}(?:\.\d{1,3}){3})\s+(\d{1,3}(?:\.\d{1,3}){3})\s+(UP|DOWN)\s+(YES|NO)\s+(.+?)(?:\s{2,}(.+))?$',
            line.strip()
        )

        if match:
            name, ip, mask, status, forward, device, flags = match.groups()
            interfaces.append({
                "Name": name.strip(),
                "IP Address": ip.strip(),
                "Subnet Mask": mask.strip(),
                "Status": status.strip(),
                "Forward": forward.strip(),
                "Device": device.strip(),
                "Flags": flags.strip() if flags else ""
            })

    return interfaces


def parse_ip_config(section_text: str) -> list[dict[str, str]]:
    """
    Parses the 'show ip config' section into a list of dictionaries.
    Filters only expected keys and removes trailing commas.
    """
    expected_keys = COLUMN_DEFINITIONS["show ip config"]
    result = {}

    for line in section_text.splitlines():
        line = line.strip()
        if not line or '=' not in line:
            continue  # skip empty lines or malformed lines
        key_part, value_part = line.split('=', 1)
        key = key_part.strip()
        value = value_part.strip().rstrip(',')  # Remove trailing commas
        if key in expected_keys:
            result[key] = value

    return [result] if result else []


def parse_ip_protocols(text):
    """
    Parses the 'show ip protocols' section and returns a list of dictionaries with column values.
    """
    result = []
    row = {}
    for line in text.splitlines():
        line = line.strip()
        if not line or "=" not in line:
            continue
        key, value = line.split("=", 1)
        key = key.strip()
        value = value.strip().rstrip(",")
        row[key] = value
    if row:
        result.append(row)
    return result


def parse_ip_dos_statistics(section_text: str) -> list[dict[str, str]]:
    stats = {}
    for line in section_text.strip().splitlines():
        line = line.strip()
        if not line or line.startswith("DoS type") or line.startswith("-"):
            continue
        if match := re.match(r"(.+?)\s{2,}(\d+)$", line):
            dos_type, attacks = match.groups()
            stats[dos_type.strip()] = attacks.strip()
    return [stats]


def parse_snmp_statistics(text: str) -> list[dict]:
    section_name = "show snmp statistics"
    columns = COLUMN_DEFINITIONS.get(section_name)
    if not columns:
        return []

    row = {}
    for line in text.splitlines():
        if "=" in line:
            key, value = line.split("=", 1)
            key = key.strip()
            value = value.strip().rstrip(",")
            if key in columns:
                row[key] = value

    # Ensure all columns are included even if not present in the output
    full_row = {col: row.get(col, "") for col in columns}

    return [full_row]  # return as a list[dict]


def parse_virtual_chassis_topology(text: str) -> list[dict]:
    section_name = "show virtual-chassis topology"
    columns = COLUMN_DEFINITIONS.get(section_name)
    if not columns:
        return []
    rows = []
    local_chassis = None
    # Extract the Local Chassis value
    for line in text.splitlines():
        match = re.match(r"Local Chassis:\s+(\d+)", line)
        if match:
            local_chassis = match.group(1)
            break
    # Regex pattern to match data lines
    pattern = re.compile(
        r"^\s*(\d+)\s+(\w+)\s+([\w+]+)\s+(\d+)\s+(\d+)\s+(\d+)\s+([0-9a-f:]{17})",
        re.IGNORECASE
    )
    for line in text.splitlines():
        match = pattern.match(line)
        if match:
            row = dict(zip(columns[:-1], match.groups()))
            row[columns[-1]] = local_chassis  # Append Local Chassis column
            rows.append(row)
    return rows


def parse_virtual_chassis_consistency(data: str) -> list[dict]:
    # Define column headers based on the provided definitions
    column_headers = COLUMN_DEFINITIONS.get("show virtual-chassis consistency", [])
    
    # Prepare the output list
    results = []
    
    # Split the input data into lines
    lines = data.strip().split("\n")
    
    # Skip the first line as it contains the legend and is not needed for parsing
    lines = lines[1:]

    # Regex pattern to match each row of data
    line_pattern = re.compile(r"^\s*(\d+)\s+(\d+)\s+([A-Za-z]+)\s+([A-Za-z0-9]+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+([A-Z]+)\s*$")

    for line in lines:
        # Match the line against the regex pattern
        match = line_pattern.match(line.strip())
        
        if match:
            # Extract the matched groups
            chas, config_chas_id, status, oper_type, oper_group, hello_interv, oper_vlan, config_vlan, license = match.groups()
            
            # Create a dictionary with the corresponding column names
            entry = {
                "Chassis ID": chas,
                "Config Chas ID": config_chas_id,
                "Status": status,
                "Oper Type": oper_type,
                "Oper Group": oper_group,
                "Hello Interv": hello_interv,
                "Oper Control Vlan": oper_vlan,
                "Config Control Vlan": config_vlan,
                "License": license
            }
            
            # Append the entry to the results list
            results.append(entry)

    # Return the results as a list of dictionaries
    
    return results


def parse_virtual_chassis_vf_link_member_port(output: str) -> List[Dict[str, str]]:
    lines = output.strip().splitlines()
    results = []
    vf_link_mode = ""

    # Extract VFLink mode
    for line in lines:
        mode_match = re.match(r"VFLink mode:\s*(\S+)", line)
        if mode_match:
            vf_link_mode = mode_match.group(1)
            break

    # Find the data section after the separator line
    data_started = False
    for line in lines:
        if re.match(r"^-+\+-+", line):
            data_started = True
            continue
        if not data_started or not line.strip():
            continue

        parts = re.split(r'\s{2,}', line.strip())
        if len(parts) == 4:
            results.append({
                "Chassis/VFLink ID": parts[0],
                "Chassis/Slot/Port": parts[1],
                "Oper": parts[2],
                "Is Primary": parts[3],
                "VFLink Mode": vf_link_mode
            })

    return results


def parse_virtual_chassis_chassis_reset_list(output: str) -> List[Dict[str, str]]:
    lines = output.strip().splitlines()
    results = []
    
    # Skip the header line
    data_started = False
    
    for line in lines:
        # Skip empty lines and the header line
        if not line.strip() or "Chas" in line:
            continue

        # Split by one or more spaces or tabs
        parts = re.split(r'\s{2,}', line.strip())
        if len(parts) == 2:
            # Remove trailing commas from the second column
            parts[1] = parts[1].rstrip(',')
            
            results.append({
                "Chas": parts[0],
                "Chassis reset list": parts[1]
            })

    return results


def parse_virtual_chassis_slot_reset_list(output: str) -> List[Dict[str, str]]:
    lines = output.strip().splitlines()
    results = []
    
    # Skip the header line
    for line in lines:
        # Skip empty lines and the header line
        if not line.strip() or "Chas" in line:
            continue

        # Split by one or more spaces or tabs
        parts = re.split(r'\s{2,}', line.strip())
        if len(parts) == 3:
            results.append({
                "Chas": parts[0],
                "Slot": parts[1],
                "Reset status": parts[2]
            })

    return results


def parse_virtual_chassis_vf_link(output: str) -> List[Dict[str, str]]:
    lines = output.strip().splitlines()
    results = []
    vf_link_mode = ""

    # Extract VFLink mode
    for line in lines:
        mode_match = re.match(r"VFLink mode:\s*(\S+)", line)
        if mode_match:
            vf_link_mode = mode_match.group(1)
            break

    # Find the data section after the separator line
    data_started = False
    for line in lines:
        if re.match(r"^-+\+-+", line):
            data_started = True
            continue
        if not data_started or not line.strip():
            continue

        # Split by two or more spaces
        parts = re.split(r'\s{2,}', line.strip())
        if len(parts) == 7:
            results.append({
                "Chassis/VFLink ID": parts[0],
                "Oper": parts[1],
                "Primary Port": parts[2],
                "Config Port": parts[3],
                "Active Port": parts[4],
                "Def Vlan": parts[5],
                "Speed Type": parts[6],
                "VFLink Mode": vf_link_mode
            })

    return results


def parse_virtual_chassis_auto_vf_link_port(output: str) -> List[Dict[str, str]]:
    lines = output.strip().splitlines()
    results = []

    # Process each line of output
    for line in lines:
        # Skip empty lines and the header line
        if not line.strip() or "Chassis/Slot/Port" in line:
            continue

        # Split by one or more spaces or tabs (to handle potential irregular spacing)
        parts = re.split(r'\s{2,}', line.strip())
        if len(parts) == 3:
            results.append({
                "Chassis/Slot/Port": parts[0],
                "Chassis/VFLink ID": parts[1],
                "VFLink member status": parts[2]
            })

    return results


def parse_virtual_chassis_neighbors(output: str) -> List[Dict[str, str]]:
    lines = output.strip().splitlines()
    results = []

    # Skip the first three lines: 2 header lines + separator
    data_lines = lines[3:]

    for line in data_lines:
        if not line.strip():
            continue

        parts = re.split(r'\s{2,}', line.strip())

        if len(parts) >= 2:
            result = {
                "Chas ID": parts[0],
                "VFL 0": parts[1],
                "VFL 1": parts[2] if len(parts) > 2 else ""
            }
            results.append(result)

    return results


def parse_debug_virtual_chassis_topology(text: str) -> List[Dict[str, str]]:
    section_name = "debug show virtual-chassis topology"
    columns = [
        "Oper Chas",
        "Role",
        "Status",
        "Config Chas ID",
        "Oper Priority",
        "Group",
        "MAC-Address",
        "System Ready",
        "Local Chassis"
    ]
    
    rows = []
    local_chassis = None

    # Extract the Local Chassis value
    for line in text.splitlines():
        match = re.match(r"Local Chassis:\s+(\d+)", line)
        if match:
            local_chassis = match.group(1)
            break

    # Regex pattern to match data lines
    pattern = re.compile(
        r"^\s*(\d+)\s+(\w+)\s+([\w+]+)\s+(\d+)\s+(\d+)\s+(\d+)\s+([0-9a-f:]{17})\s+(\w+)",
        re.IGNORECASE
    )

    for line in text.splitlines():
        match = pattern.match(line)
        if match:
            row = dict(zip(columns[:-1], match.groups()))
            row["Local Chassis"] = local_chassis
            rows.append(row)

    return rows


def parse_debug_virtual_chassis_status(output: str) -> List[Dict[str, str]]:
    columns = COLUMN_DEFINITIONS["debug show virtual-chassis status"]

    rows = []

    # Regex pattern to capture the 6 fields, allowing for spacing in "Parameter"
    pattern = re.compile(
        r"^\s*(\d+)\s+(\w+)\s+(.+?)\s{2,}(\S+)\s+(\S+)\s+(\S+)\s*$"
    )

    for line in output.strip().splitlines():
        match = pattern.match(line)
        if match:
            row = dict(zip(columns, match.groups()))
            rows.append(row)

    return rows


def parse_debug_virtual_chassis_connection(output: str) -> List[Dict[str, str]]:
    columns = COLUMN_DEFINITIONS["debug show virtual-chassis connection"]

    rows = []

    # Regex to match the expected row format
    pattern = re.compile(
        r"^\s*(\d+)\s+([0-9a-f:]{17})\s+([\d.]+)\s+([\d.]+)\s+(\w+)\s*$",
        re.IGNORECASE
    )

    for line in output.strip().splitlines():
        match = pattern.match(line)
        if match:
            row = dict(zip(columns, match.groups()))
            rows.append(row)

    return rows


def parse_show_cloud_agent_status(output: str) -> List[Dict[str, str]]:
    result = {}
    
    # Match lines in the format: Key : Value (optionally ending with a comma)
    pattern = re.compile(r"^\s*(.+?)\s*:\s*(.+?)(?:,)?\s*$")
    
    for line in output.strip().splitlines():
        match = pattern.match(line)
        if match:
            key = match.group(1).strip()
            value = match.group(2).strip()
            result[key] = value

    return [result]


def parse_pkgmgr(output: str) -> List[Dict[str, str]]:
    columns = COLUMN_DEFINITIONS["show pkgmgr"]
    rows = []

    for line in output.strip().splitlines():
        # Skip legend, column header, and separator lines
        if "indicates" in line or "Name" in line or set(line.strip()) <= {"-", "+"}:
            continue

        parts = line.strip().split(None, 3)
        if len(parts) == 4:
            row = dict(zip(columns, parts))
            rows.append(row)

    return rows


def parse_appmgr(output: str) -> List[Dict[str, str]]:
    columns = COLUMN_DEFINITIONS["show appmgr"]    
    rows = []

    for line in output.strip().splitlines():
        # Skip legend, header, and separator lines
        if "indicates" in line or "Application" in line or set(line.strip()) <= {"-", "+"}:
            continue

        parts = line.strip().split(None, 4)  # Split into max 5 fields
        if len(parts) == 5:
            row = dict(zip(columns, parts))
            rows.append(row)

    return rows


def parse_naas_license(output: str) -> List[Dict[str, str]]:
    columns = COLUMN_DEFINITIONS["show naas license"]    
    rows: List[Dict[str, str]] = []

    for line in output.strip().splitlines():
        # Skip headers, separators, and legend lines
        if "Serial" in line or "VC" in line or set(line.strip()) <= {"-", "+"}:
            continue

        parts = line.strip().split(None, 8)  # maxsplit=8 for 9 columns
        if len(parts) == 9:
            row = dict(zip(columns, parts))
            rows.append(row)

    return rows


def parse_naas_agent_status(output: str) -> List[Dict[str, str]]:
    columns = COLUMN_DEFINITIONS["show naas-agent status"]    
    data: Dict[str, str] = {}

    for line in output.strip().splitlines():
        if ":" not in line:
            continue
        key, value = line.split(":", 1)
        key = key.strip()
        value = value.strip().rstrip(",")
        if key in columns:
            data[key] = value

    return [data] if data else []


def parse_capability_naas(output: str) -> List[Dict[str, str]]:
    columns = COLUMN_DEFINITIONS["debug show capability naas"]    
    rows: List[Dict[str, str]] = []

    for line in output.strip().splitlines():
        # Skip legend, headers, and separators
        if line.strip().startswith("+") or "NAAS" in line:
            continue

        # Split line by pipe delimiter
        parts = [part.strip() for part in line.strip().strip('|').split('|')]
        if len(parts) == len(columns):
            row = dict(zip(columns, parts))
            rows.append(row)

    return rows


def parse_ntp_server_status(output: str) -> List[Dict[str, str]]:
    columns = COLUMN_DEFINITIONS["show ntp server status"]
    
    # Initialize a dictionary to store key-value pairs
    data = {}

    # Split the output by lines
    lines = output.strip().splitlines()

    # Iterate through each line
    for line in lines:
        # Skip lines that don't contain '=' or are empty
        if '=' not in line:
            continue
        
        # Split the line into key and value based on '='
        parts = line.split("=", 1)
        
        if len(parts) == 2:
            key = parts[0].strip()
            value = parts[1].strip().rstrip(',')  # Remove trailing commas
            
            # Only add to dictionary if the key is in the columns
            if key in columns:
                data[key] = value

    # Return the parsed data as a list of dictionaries
    return [data] if data else []


def parse_ntp_status(output: str) -> List[Dict[str, str]]:
    columns = COLUMN_DEFINITIONS["show ntp status"]    
    data = {}

    for line in output.strip().splitlines():
        if ':' not in line:
            continue

        key, value = line.split(":", 1)
        key = key.strip()
        value = value.strip().rstrip(',')

        if key in columns:
            data[key] = value

    return [data] if data else []


def parse_ntp_keys(output: str) -> List[Dict[str, str]]:
    columns = COLUMN_DEFINITIONS["show ntp keys"]    
    rows = []

    for line in output.strip().splitlines():
        # Skip header and separator lines
        if line.strip().startswith("Key") or "-" in line or "+" in line:
            continue

        parts = line.strip().split(None, 1)
        if len(parts) == 2:
            row = dict(zip(columns, parts))
            rows.append(row)

    return rows


def parse_capability_profile(output: str) -> List[Dict[str, str]]:
    columns = COLUMN_DEFINITIONS["show capability profile"]    

    data = {}

    for line in output.strip().splitlines():
        if ':' not in line:
            continue

        key, value = line.split(':', 1)
        key = key.strip().replace("  ", " ")  # Normalize spacing
        value = value.strip().rstrip(',')

        if key in columns:
            data[key] = value

    return [data] if data else []


PARSERS = {
    "show hardware-info": parse_hardware_info,
    "show chassis": parse_chassis,
    "show module long": parse_module_long,
    "show transceivers": parse_transceivers,
    "show fan": parse_fan,
    "show powersupply": parse_powersupply,
    "show temperature": parse_temperature,
    "show system": parse_system,
    "show running-directory": parse_running_directory,
    "show microcode": parse_microcode,
    "show license-info": parse_license_info,
    "show lldp remote-system": parse_lldp_remote_system,
    "show aaa authentication": parse_aaa_authentication,
    "show health": parse_health,
    "show health all cpu": parse_health_all_cpu,
    "show vlan": parse_vlan,
    "show spantree": parse_spantree,
    "show spantree ports active": parse_spantree_ports_active,
    "show interfaces status": parse_interfaces_status,
    "show interfaces counters": parse_interfaces_counters,
    "show ip interface": parse_ip_interface,
    "show ip config": parse_ip_config,
    "show ip protocols": parse_ip_protocols,
    "show ip dos statistics": parse_ip_dos_statistics,
    "show snmp statistics": parse_snmp_statistics,
    "show virtual-chassis topology": parse_virtual_chassis_topology,
    "show virtual-chassis consistency": parse_virtual_chassis_consistency,
    "show virtual-chassis vf-link member-port": parse_virtual_chassis_vf_link_member_port,
    "show virtual-chassis chassis-reset-list": parse_virtual_chassis_chassis_reset_list,
    "show virtual-chassis slot-reset-list": parse_virtual_chassis_slot_reset_list,
    "show virtual-chassis vf-link": parse_virtual_chassis_vf_link,
    "show virtual-chassis auto-vf-link-port": parse_virtual_chassis_auto_vf_link_port,
    "show virtual-chassis neighbors": parse_virtual_chassis_neighbors,
    "debug show virtual-chassis topology": parse_debug_virtual_chassis_topology,
    "debug show virtual-chassis status": parse_debug_virtual_chassis_status,
    "debug show virtual-chassis connection": parse_debug_virtual_chassis_connection,
    "show cloud-agent status": parse_show_cloud_agent_status,
    "show pkgmgr": parse_pkgmgr,
    "show appmgr": parse_appmgr,
    "show naas license": parse_naas_license,
    "show naas-agent status": parse_naas_agent_status,
    "debug show capability naas": parse_capability_naas,
    "show ntp server status": parse_ntp_server_status,
    "show ntp status": parse_ntp_status,
    "show ntp keys": parse_ntp_keys,
    "show capability profile": parse_capability_profile,
}

def extract_section(full_text: str, section_name: str) -> str:
    pattern = fr"#+\s*{re.escape(section_name)}\s*#+\n(.*?)(?=\n#+|\Z)"
    match = re.search(pattern, full_text, re.DOTALL)
    return match.group(1).strip() if match else ""


def parse_sections(full_text: str) -> Dict[str, List[Dict[str, str]]]:
    parsed_output = {}
    for section_name, parser in PARSERS.items():
        # Special handling for combined microcode sections
        if (section_name == "show microcode"):
            combined_microcode_data = []
            for location in ["certified", "working", "loaded"]:
                specific_section_name = f"show microcode {location}"
                raw_text = extract_section(full_text, specific_section_name)
                if raw_text:
                    parsed = parser(raw_text, location)
                    if parsed:
                        combined_microcode_data.extend(parsed)
            if combined_microcode_data:
                parsed_output["show microcode"] = combined_microcode_data
        else:
            raw_text = extract_section(full_text, section_name)
            if raw_text:
                parsed_output[section_name] = parser(raw_text)
    return parsed_output


def export_to_csv(parsed_data: Dict[str, List[Dict[str, str]]]):
    timestamp = datetime.now().strftime("%Y-%m-%d_%H%M%S")
    filename = f"parsed_sections_{timestamp}.csv"
    
    with open(filename, mode='w', newline='', encoding='utf-8') as file:
        writer = csv.writer(file)
        
        for idx, (section_name, rows) in enumerate(parsed_data.items()):
            if not rows:
                writer.writerow([f"{section_name}"])
                headers = COLUMN_DEFINITIONS.get(section_name)
                writer.writerow(headers)
                if idx < len(parsed_data) - 1:
                    writer.writerows([[]] * 2)
                continue

            writer.writerow([f"{section_name}"])

            headers = COLUMN_DEFINITIONS.get(section_name, list(rows[0].keys()))
            writer.writerow(headers)

            for row in rows:
                row_out = []
                for col in headers:
                    val = row.get(col, "")
                    # 🛡️ Protect any value with "/" (like 1/1) from Excel auto-formatting
                    if isinstance(val, str) and "/" in val and not val.startswith("=\""):
                        val = f"=\"{val}\""
                    row_out.append(val)
                writer.writerow(row_out)

            # Add 2 blank lines between sections
            if idx < len(parsed_data) - 1:
                writer.writerows([[]] * 2)

    print(f"✅ CSV exported to {filename}")


def main():
    log_file = "tech_support.log"
    if not os.path.isfile(log_file):
        print(f"❌ File not found: {log_file}")
        return

    with open(log_file, encoding='utf-8') as f:
        full_text = f.read()

    parsed = parse_sections(full_text)
    if not parsed:
        print("⚠️ No recognizable sections found.")
    else:
        export_to_csv(parsed)

if __name__ == "__main__":
    main()



from .extract_ts_tar import main as extract_all_cwd
from .extract_ts_tar import extract_tar_archive, decompress_gz_file, extract_archives

# from .legacy.extracttar import (
#     extract_tar_files, 
#     extract_gz_files, 
#     main
# )
__all__ = [
    'extract_tar_archive', 
    'decompress_gz_file', 
    'extract_archives',
    'extract_all_cwd',
]



from . import aos
from . import extracttar
from . import hmon
from . import log_analyzer
#from . import tsbuddy_menu # Avoid Python script import issue 
from . import tslog2csv
from . import utils

__all__ = [
    'aos',
    'extracttar',
    'hmon',
    'log_analyzer',
    #'tsbuddy_menu',
    'tslog2csv',
    'utils',
]


import paramiko
from getpass import getpass
import time
import re
import json
import os
import sys
import subprocess

image_map = {
    "nandi_sim": ["Nossim.img"],
    "everest": ["Uos.img"],
    "medora_sim64": ["Mossim.img", "Menisim.img"],
    "tor": ["Tos.img"],
    "vindhya": ["Nos.img"],
    "medora": ["Mos.img", "Meni.img", "Mhost.img"],
    "yukon": ["Yos.img"],
    "shasta": ["Uos.img"],
    "aravalli": ["Nosa.img"],
    "shasta_n": ["Uosn.img"],
    "whitney": ["Wos.img"],
    "nandi": ["Nos.img"],
    "whitney_sim": ["Wossim.img"],
    "kailash": ["Kaos.img"]
}

model_family_map = {
 '6360': 'aravalli',
 '6465': 'vindhya',
 '6560': 'nandi',
 '6570M': 'whitney',
 '6860': 'shasta',
 '6860N': 'shasta_n',
 '6865': 'everest',
 '6870': 'kailash',
 '6900 Tor': 'tor',
 '6900 Yukon': 'yukon',
 '9900': 'medora'
 }

model_6900_variants = {
    "6900 Yukon":"T48C6/V72/V48C8/X48C6", 
    "6900 Tor":"X20/X40/X72/T20/T40/Q32"
    }

family_model_image_map = {
  "everest": {
    "models": ["6865", "6865"],
    "Img": "Uos.img"
  },
  "tor": {
    "models": ["6900 Tor", "X20/X40/X72/T20/T40/Q32"],
    "Img": "Tos.img"
  },
  "vindhya": {
    "models": ["6465", "6465"],
    "Img": "Nos.img"
  },
  "medora": {
    "models": ["9900", "9900"],
    "Img": "Mos.img"
  },
  "yukon": {
    "models": ["6900 Yukon", "T48C6/V72/V48C8"],
    "Img": "Yos.img"
  },
  "shasta": {
    "models": ["6860", "6860"],
    "Img": "Uos.img"
  },
  "aravalli": {
    "models": ["6360", "6360"],
    "Img": "Nosa.img"
  },
  "shasta_n": {
    "models": ["6860N", "6860N"],
    "Img": "Uosn.img"
  },
  "whitney": {
    "models": ["6570M", "6570M"],
    "Img": "Wos.img"
  },
  "nandi": {
    "models": ["6560", "6560"],
    "Img": "Nos.img"
  },
  "kailash": {
    "models": ["6870", "6870"],
    "Img": "Kaos.img"
  }
}

def safe_password_prompt(prompt="Password: ", fallback="switch"):
    try:
        if sys.stdin.isatty():
            return getpass.getpass(prompt) or fallback
        else:
            # Try manual stty fallback
            try:
                print("Incompatible terminal detected. Password will not be hidden.\n")
                sys.stdout.write(prompt)
                sys.stdout.flush()
                subprocess.check_call(["stty", "-echo"])
                password = input("")
            finally:
                subprocess.call(["stty", "echo"])
                sys.stdout.write("\n")
            return password or fallback
    except Exception:
        # Final fallback
        #print("Falling back to visible input.")
        return input(prompt) or fallback

# Load GA index once at the module level
ga_index_path = os.path.join(os.path.dirname(__file__), "ga_index.json")
with open(ga_index_path) as f:
    ga_index = json.load(f)

def wait_for_shell(shell, timeout=120):
    """Read output from shell until timeout or prompt returns."""
    shell.settimeout(2)
    output = ""
    start = time.time()
    while time.time() - start < timeout:
        try:
            chunk = shell.recv(4096).decode("utf-8")
            output += chunk
            # Break if we detect shell prompt (e.g., ending in # or $)
            if any(p in chunk for p in ["# ", "#->", "$ ", ">", "login: "]):
                break
        except Exception:
            time.sleep(1)
    return output

def prompt_initial_aos_version():
    """Prompts the user for the full AOS version string."""
    return input("\nEnter AOS version string (e.g., 8.9.221.R03): ").strip()

def parse_aos_version_string(version_string):
    """
    Parses the AOS version string into major, build, and release components.
    Returns a dictionary with 'major', 'build', and 'release' keys.
    Values can be None if not found in the string.
    """
    regex_pattern = r'^(\d+\.\d+)(?:\.(\d+))?(?:\.?([Rr]\d+))?$'
    match = re.match(regex_pattern, version_string)

    if match:
        major = match.group(1)
        build = match.group(2) if match.group(2) else None  # Can be None
        release_raw = match.group(3).upper() if match.group(3) else None # Can be None
        # Conditional formatting
        release = None
        if release_raw:
            if major.startswith("8."):
                # Convert to "R03" format
                number_part = release_raw[1:].zfill(2)  # Pad to 2 digits
                release = "R" + number_part
            else:
                release = release_raw
        return {"major": major, "build": build, "release": release}
    else:
        # If the basic major.minor pattern isn't found, return None for all
        return {"major": None, "build": None, "release": None}

def validate_and_complete_version_parts(parsed_parts):
    """
    Validates parsed version sections. If a section (build or release)
    is missing (None), prompts the user for it.
    Assumes 'major' will always be present if initial parsing was somewhat successful.
    """
    major = parsed_parts.get("major") # Should already be there from a successful parse
    build = parsed_parts.get("build")
    release = parsed_parts.get("release")
    if major is None:
        # This indicates a fundamental failure in parsing the initial string.
        # The main loop should handle this by re-prompting for the full string.
        # However, if we wanted to be extremely robust and prompt for major here:
        while not major:
            major = input("Enter AOS Major version (e.g., 8.9): ").strip()
        # For this exercise, we assume parse_aos_version_string got the major if input was valid.
        #pass # Major should be handled by the calling function based on parse_aos_version_string output
    if build is None:
        build = input("Enter AOS Build Number (e.g., 221): ").strip() or "221" # Default if empty
    if release is None:
        release = input("Enter AOS Release (e.g., R03): ").strip() or "R03" # Default if empty
    return {"major": major, "build": build, "release": release}

def get_aos_version_orchestrator():
    """Orchestrates prompting, parsing, and validation to get the full AOS version."""
    while True:
        initial_version_string = prompt_initial_aos_version()
        parsed_components = parse_aos_version_string(initial_version_string)
        if not parsed_components.get("major"):
            print("Invalid format. Major version (e.g., X.Y) could not be parsed. Please try again.")
            continue # Re-prompt for the full string
        # Now, parsed_components["major"] is guaranteed to be something.
        # Fill in any missing optional parts (build, release)
        completed_components = validate_and_complete_version_parts(parsed_components)
        aos_major = completed_components["major"]
        aos_build = completed_components["build"]
        aos_release = completed_components["release"]
        full_version = f"{aos_major}.{aos_build}.{aos_release}"
        confirm = input(f"Confirm full AOS version string [{full_version}] [y]/n: ").strip().lower() or "y"
        if confirm == "y":
            return aos_major, aos_build, aos_release
        # If not 'y', the loop will restart, prompting for the initial string again.

def get_aos_version_simple():
    while True:
        """Prompts for AOS version components and confirms the full version."""
        aos_major = input("Enter AOS Major Version (e.g., 8.9) [8.9]: ") or "8.9"
        aos_build = input("Enter AOS Build Number (e.g., 221) [221]: ") or "221"
        aos_release = input("Enter AOS Release (e.g., R03) [R03]: ") or "R03"
        full_version = f"{aos_major}.{aos_build}.{aos_release}"
        confirm = input(f"Confirm full AOS version string [{full_version}] [y]/n: ").strip().lower() or "y"
        if confirm == "y":
            return aos_major, aos_build, aos_release
        # If not 'y', the loop will restart, prompting for the initial string again.

def get_ga_build(version, family):
    try:
        build = ga_index[version][family]
        if build in ('', 'N/S', 'N/A', 'UNK'):
            raise ValueError(f"No GA build available for version {version} and family {family}. Build returned: {build}")
        return build
    except KeyError:
        raise ValueError(f"No GA build found for version {version} and family {family}")

def lookup_ga_build():
    """Allows the user to look up GA builds with an option to upgrade OmniSwitch."""
    print("\n----- Models -----")
    print("---- Model : Family ----")
    for model, family in model_family_map.items():
        max_key_length = max(len(k) for k in model_family_map)
        print(f"{model:<{max_key_length}} : {family}")
    print("\n----- 6900 Variants -----")
    print("-- Variant : Models --")
    for model, variants in model_6900_variants.items():
        max_key_length = max(len(k) for k in model_6900_variants)
        print(f"{model:<{max_key_length}} : {variants}")
    print("\nLookup the GA build by providing a model or device IP...")
    while True:
        user_input = input("Enter a model name (e.g., 6900 Yukon) or device IP (e.g., 192.168.1.1) [exit]: ").strip()
        if not user_input or user_input.lower() == 'exit':
            print("Canceling GA build lookup...")
            break
        # Use regex to check for IP (contains . or :)
        if re.search(r'[.:]', user_input):
            # Assume IP, prompt for credentials
            ip = user_input
            username = input(f"Enter username for {ip} [admin]: ") or "admin"
            password = getpass(f"Enter password for {ip} [switch]: ") or "switch"
            host = {"ip": ip, "username": username, "password": password}
            family = get_family_from_ip(host)
            if not family:
                print(f"Could not determine family from IP {ip}.")
                continue
            print(f"Family for {ip}: {family}")
        else:
            # Assume model
            model = user_input
            family = model_family_map.get(model)
            if not family:
                print(f"Unknown model: {model}. Please try again.")
                continue
            print(f"Family for model {model}: {family}")
        ga_prompt_ver = input("Provide the AOS version & Release for GA lookup (e.g., 8.10R02) [exit]: ").strip().upper() or None
        if not ga_prompt_ver or ga_prompt_ver.lower() == 'exit':
            print("Lookup canceled.")
            continue
        gadl = input(f"Would you like to download to OmniSwitch? [y]/n: ").strip().lower() or "y"
        try:
            ga_ver, ga_release = ga_prompt_ver.split('R') if ga_prompt_ver else (None, None)
            ga_release = "R" + ga_release  # Add "R" to the beginning of the ga_release string
            found_ga_build = f"{ga_ver}{get_ga_build(ga_prompt_ver, family)}.{ga_release}"
            print("GA Build: ", found_ga_build)
            if gadl == "y":
                aosup(found_ga_build=found_ga_build)
        except Exception as e:
            print(f"Error looking up GA build: {e}")

def collect_hosts():
    """Collects device details from the user and returns a list of hosts."""
    hosts = []
    print("\nEnter device details. Press Enter without an IP to finish.")
    while True:
        ip = input("Enter device IP: ").strip()
        if not ip:
            break
        username = input(f"Enter username for {ip} [admin]: ") or "admin"
        password = safe_password_prompt(f"Enter password for {ip} [switch]: ")
        #password = getpass(f"Enter password for {ip} [switch]: ") or "switch"
        hosts.append({"ip": ip, "username": username, "password": password})
    return hosts


def get_platform_family(shell):
    """Extracts the platform family from the shell prompt."""
    shell.send("echo $PS1\n")
    time.sleep(1)
    output = shell.recv(2048).decode("utf-8")
    lines = output.strip().splitlines()
    ps1 = lines[-1] if lines else ""
    family = ps1.split()[0].lower() if ps1 else None
    return family


def aosup(found_ga_build=None):
    """Prompt the user for folder name and reload option, then execute main with those arguments."""
    folder_name = input("Which folder name would you like to download to? [working]: ").strip() or "working"
    reload_choice = input("Would you like to reload when finished? [y]/n: ").strip().lower() or "y"
    reload_when_finished = reload_choice == "y"
    main(folder_name=folder_name, reload_when_finished=reload_when_finished, found_ga_build=found_ga_build)
    #if reload_when_finished:
    #    print(f"Reloading from folder '/flash/{folder_name}'...")
    #else:
    #    print(f"Download completed. Files are in the '/flash/{folder_name}' directory.")
    #return folder_name, reload_when_finished


def download_images_for_host(host, aos_major, aos_build, aos_release, image_map, base_ip, base_dir, folder_name=None, reload_when_finished=False):
    ip = host["ip"]
    print(f"\nConnecting to {ip}...")
    try:
        # Connect via SSH
        client = paramiko.SSHClient()
        client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        client.connect(ip, username=host["username"], password=host["password"], timeout=10)
        shell = client.invoke_shell()
        shell.send("su\n")
        time.sleep(1)
        shell.recv(1024)  # No password needed
        family = get_platform_family(shell)
        if not family or family not in image_map:
            print(f"[{ip}] Unknown or missing platform family: '{family}'")
            client.close()
            return
        print(f"[{ip}] Platform family: {family}")
        images = image_map[family]
        aos_major_fmt = aos_major.replace('.', '_')
        image_path = f"{base_dir}/OS_{aos_major_fmt}_{aos_build}_{aos_release}/{family}/Release/"
        for img in images:
            url = f"{base_ip}{image_path}{img}"
            if folder_name:
                cmd = f"curl -kL \"{url}\" --output /flash/{folder_name}/{img}"
                shell.send(cmd + "\n")
                print(f"[{ip}] Downloading {img}...")
                download_output = wait_for_shell(shell)
                print(f"[{ip}] Downloaded {img} to /flash/{folder_name}/")
            else:
                cmd = f"curl -kL \"{url}\" --output /flash/{img}"
                shell.send(cmd + "\n")
                print(f"[{ip}] Downloading {img}...")
                download_output = wait_for_shell(shell)
                print(f"[{ip}] Downloaded {img} to /flash/")
        if reload_when_finished and folder_name:
            shell.send("exit\n")
            time.sleep(4)
            shell.send(f"echo 'y' | reload from {folder_name} no rollback-timeout\n")
            time.sleep(4)
            print(f"[{ip}] Started reload from /flash/{folder_name}/")
        client.close()
    except Exception as e:
        print(f"[{ip}] ERROR: {e}")
    finally:
        try:
            client.close()
        except:
            pass

def get_family_from_filename(filename: str) -> str | None:
    """Infers the family by finding the provided filename in the mapping."""
    for family, filenames_list in image_map.items():
        if filename in filenames_list:
            return family
    return None

def get_filenames_for_family(family: str) -> list[str] | None:
    """Retrieves the list of software filenames for a given family."""
    return image_map.get(family)

def get_family_from_ip(host: str) -> str | None:
    """
    Connects to a device via SSH to determine its model and then its family.
    """
    ip = host["ip"]
    print(f"\nConnecting to {ip}...")
    try:
        # Connect via SSH
        client = paramiko.SSHClient()
        client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        client.connect(ip, username=host["username"], password=host["password"], timeout=10)
        shell = client.invoke_shell()
        shell.send("su\n")
        time.sleep(1)
        shell.recv(1024)  # No password needed
        family = get_platform_family(shell)
        if not family or family not in image_map:
            print(f"[{ip}] Unknown or missing platform family: '{family}'")
            client.close()
            return
        print(f"[{ip}] Platform family: {family}")
        client.close()
        return family
    except Exception as e:
        print(f"[{ip}] ERROR: {e}")
    finally:
        try:
            client.close()
        except:
            pass

def main(folder_name=None, reload_when_finished=False, found_ga_build=None):
    print("\nNote: you can lookup the GA build with aosdl-ga CLI command.\n")
    if not found_ga_build:
        aos_major, aos_build, aos_release = get_aos_version_simple()
    else:
        parts = found_ga_build.split('.')
        aos_major, aos_build, aos_release = [".".join(parts[:2]), parts[2], parts[3]]
    print(f"Using AOS Version: {aos_major}.{aos_build}.{aos_release}")
    # Image mapping, moved to global scope for reusability
    base_ip = "http://10.46.4.37"
    base_dir = "/bop/images"
    hosts = collect_hosts()
    for host in hosts:
        download_images_for_host(host, aos_major, aos_build, aos_release, image_map, base_ip, base_dir, folder_name, reload_when_finished)

if __name__ == "__main__":
    main()




import importlib.metadata as metadata
import urllib.request
import urllib.error
import json
import subprocess
import sys
import os
import re
import time

# --- Saved Parameters ---

ENV_FILE = os.path.join(os.path.expanduser("~"), ".tsbuddy_settings")

def load_env_file():
    """Load key-value pairs from .env into os.environ"""
    if os.path.exists(ENV_FILE):
        with open(ENV_FILE) as f:
            for line in f:
                if line.strip() and not line.startswith("#"):
                    key, sep, value = line.strip().partition("=")
                    if sep:  # Only set if '=' was found
                        os.environ[key] = value  # override to allow updates


def set_env_variable(key, value):
    """Set or update a key=value in .env file."""
    lines = []
    found = False
    if os.path.exists(ENV_FILE):
        with open(ENV_FILE) as f:
            for line in f:
                if line.strip().startswith(f"{key}="):
                    lines.append(f"{key}={value}\n")
                    found = True
                else:
                    lines.append(line)
    if not found:
        lines.append(f"{key}={value}\n")
    with open(ENV_FILE, "w") as f:
        f.writelines(lines)

# Load .env into environment
load_env_file()

# --- Version Handling Functions ---

def get_installed_version(package_name):
    try:
        return metadata.version(package_name)
    except metadata.PackageNotFoundError:
        return None

def get_latest_version(package_name):
    url = f"https://pypi.org/pypi/{package_name}/json"
    try:
        with urllib.request.urlopen(url) as resp:
            return json.load(resp)["info"]["version"]
    except Exception as e:
        print(f"Error fetching latest version for '{package_name}': {e}")
        return None

def get_pypi_description(package_name, limit=3):
    """Fetch the package description from PyPI JSON API."""
    url = f"https://pypi.org/pypi/{package_name}/json"
    try:
        with urllib.request.urlopen(url) as resp:
            data = json.load(resp)
            content = data["info"].get("description")
    except Exception as e:
        print(f"Error fetching description for '{package_name}': {e}")
        return None
    
    # Extract the full changelog section
    match = re.search(r"##\s*Changelog\s*([\s\S]*?)(?:\n## |\Z)", content, re.IGNORECASE)
    if not match:
        return None

    changelog_section = match.group(1).strip()

    # Split by full changelog entries (keep the delimiter)
    entries = re.findall(r"(###\s.*?)(?=\n### |\Z)", changelog_section, re.DOTALL)
    
    if not entries:
        return None

    # Take the most recent N entries
    latest_entries = entries[:limit]
    return "\n\n".join(entry.strip() for entry in latest_entries)

# --- Changelog Fetching Function ---

def fetch_changelog(limit=3):
    changelog_url = "https://raw.githubusercontent.com/bgbyte/tsbuddy/main/README.md"
    try:
        with urllib.request.urlopen(changelog_url) as resp:
            content = resp.read().decode("utf‑8")
    except urllib.error.HTTPError as e:
        print(f"Error retrieving changelog: {e}")
        return None

    # Extract the full changelog section
    match = re.search(r"##\s*Changelog\s*([\s\S]*?)(?:\n## |\Z)", content, re.IGNORECASE)
    if not match:
        return None

    changelog_section = match.group(1).strip()

    # Split by full changelog entries (keep the delimiter)
    entries = re.findall(r"(###\s.*?)(?=\n### |\Z)", changelog_section, re.DOTALL)
    
    if not entries:
        return None

    # Take the most recent N entries
    latest_entries = entries[:limit]
    return "\n\n".join(entry.strip() for entry in latest_entries)



# --- Update Function ---

def update_package(package_name):
    print(f"\n🔄 Updating '{package_name}'...\n")
    subprocess.check_call([sys.executable, "-m", "pip", "install", "--upgrade", package_name])
    print(f"\n✅ '{package_name}' updated.\n")

def update_package_safe(package_name, current_version=None):
    import subprocess, sys, os
    import tempfile

    # Path to re-launch after upgrade
    #relaunch_cmd = [sys.executable, "-m", "tsbuddy"]
    #relaunch_cmd = ["tsbuddy"]
    
    # Path to the temporary updater script
    updater_path = os.path.join(tempfile.gettempdir(), "_tsbuddy_updater.py")

    print(f"\n🔄 Preparing to update '{package_name}'...")

    # Write a temporary updater script
    updater_script = f"""
import time
import subprocess
import sys

print("Waiting for current process to exit...")
time.sleep(2)  # Give time for the main script to exit

subprocess.check_call([r"{sys.executable}", "-m", "pip", "install", "--upgrade", "{package_name}", "--trusted-host", "pypi.org", "--trusted-host", "files.pythonhosted.org"])
print("\\n"+("#"*15))
print("Please report any bugs to Brian.")
print("If there is an issue, you can revert to your previous version using: ")
print("pip install tsbuddy=={current_version}")
print("#"*15,"\\n")
time.sleep(5)
print("\\n* Upgrade complete. You can now rerun tsbuddy.")
"""

    #updater_path = os.path.join(os.getcwd(), "_tsbuddy_updater.py")

    with open(updater_path, "w") as f:
        f.write(updater_script)

    # Launch the updater in a new process
    subprocess.Popen([sys.executable, updater_path], close_fds=True)

    if current_version:
        set_env_variable("TSBUDDY_PREVIOUS_VERSION", current_version)
    set_env_variable("TSBUDDY_IGNORE_VERSION", "")  # Clear ignore version if set
    print("Exiting to allow upgrade to complete...")
    sys.exit(0)

def downgrade_package_safe(package_name, previous_version):
    import subprocess, sys, os
    import tempfile

    # Path to re-launch after upgrade
    #relaunch_cmd = [sys.executable, "-m", "tsbuddy"]
    #relaunch_cmd = ["tsbuddy"]
    
    # Path to the temporary updater script
    updater_path = os.path.join(tempfile.gettempdir(), "_tsbuddy_updater.py")

    print(f"\n🔄 Preparing to downgrade '{package_name}'...")

    # Write a temporary updater script
    updater_script = f"""
import time
import subprocess
import sys

print("Waiting for current process to exit...")
time.sleep(2)  # Give time for the main script to exit

subprocess.check_call([r"{sys.executable}", "-m", "pip", "install", "{package_name}=={previous_version}", "--trusted-host", "pypi.org", "--trusted-host", "files.pythonhosted.org"])
print("\\n"+("#"*15))
print("Important: Please report any bugs to Brian.")
print("#"*15,"\\n")
time.sleep(5)
print("\\n* Downgrade complete. You can now rerun tsbuddy.")
"""

    #updater_path = os.path.join(os.getcwd(), "_tsbuddy_updater.py")

    with open(updater_path, "w") as f:
        f.write(updater_script)

    # Launch the updater in a new process
    subprocess.Popen([sys.executable, updater_path], close_fds=True)

    print("Exiting to allow downgrade to complete...")
    sys.exit(0)

def downgrade_to_previous_version(package="tsbuddy"):
    """
    Downgrade tsbuddy to the previous version found in the environment variable TSBUDDY_PREVIOUS_VERSION.
    """
    previous_version = os.environ.get("TSBUDDY_PREVIOUS_VERSION")
    if not previous_version:
        print("No previous version found in environment variable TSBUDDY_PREVIOUS_VERSION loaded from ~/.tsbuddy_settings")
        print("If you want to downgrade, please specify the version manually, e.g.: 0.0.23")
        #print("pip install tsbuddy==0.0.23")
        previous_version = input("Enter the version to downgrade to: ").strip()
        if not previous_version:
            return
    print(f"Enter version to downgrade to. By default, previous version: {previous_version}")
    previous_version = input(f"Downgrade to version [{previous_version}]: ").strip() or previous_version
    print(f"\n⏬ Downgrading 'tsbuddy' to version: {previous_version} ...\n")
    try:
        downgrade_package_safe(package, previous_version)
        print(f"\n✅ 'tsbuddy' downgraded to version {previous_version}.")
        set_env_variable("TSBUDDY_IGNORE_VERSION", "")  # Clear ignore version if set
    except Exception as e:
        print(f"Error downgrading to version {previous_version}: {e}")

def choice_form():
    choice = input("Do you want to upgrade or downgrade? [U/d]: ").strip().lower()
    if choice == 'u':
        update_package_safe("tsbuddy", current_version=get_installed_version("tsbuddy"))
    elif choice == 'd':
        downgrade_to_previous_version()
    else:
        return None


# --- Main Logic ---

def main():
    package = "tsbuddy"
    #print(f"\nChecking '{package}'...\n")

    current_version = get_installed_version(package)
    latest_version = get_latest_version(package)

    if latest_version is None:
        print(f"⚠ Could not determine the latest version of '{package}'.")
        return

    # Check if user wants to ignore this version
    ignore_version = os.environ.get("TSBUDDY_IGNORE_VERSION")
    if latest_version == ignore_version != current_version:
        print(f"⚠  Old version {current_version}\n⬆️ Upgrade {package} to {latest_version} manually using: \npip cache purge\npip install --upgrade {package}")
        time.sleep(1)
        return

    show_changelog = False

    if current_version is None:
        print(f"📦 '{package}' is not installed. Latest available: {latest_version}")
        show_changelog = True
    elif current_version != latest_version:
        show_changelog = True
    else:
        print(f"✅ '{package}' is up to date ({current_version})")
        return

    # Show changelog if available
    if show_changelog:
        # print("\n--- 📦 PyPI Description ---\n")
        # description = get_pypi_description(package)
        # if description:
        #     print(description)
        # else:
        #     print("No description found on PyPI.")
        print("\n--- 📄 Latest Changelog Entries ---\n")
        #changelog = fetch_changelog(limit=3)
        changelog = get_pypi_description(package)
        if changelog:
            print(changelog)
            print("\nView full history at: https://github.com/bgbyte/tsbuddy#changelog")
            print(f"\n⬇ A newer version of '{package}' is available: {current_version} → {latest_version}")
        else:
            print("No changelog found.")

    # Ask for update
    confirm = input(f"Do you want to update '{package}' to {latest_version}? [y/N], 'N' will skip future prompts for {latest_version}: ").strip().lower()
    if confirm == 'y':
        update_package_safe(package, current_version)
    else:
        set_env_variable("TSBUDDY_IGNORE_VERSION", latest_version)
        print(f"Skipping version {latest_version}.")
        print("If you want to update before the next release, run `pip install --upgrade tsbuddy` manually.")
        print("Loading tsbuddy...")
        time.sleep(1)

if __name__ == "__main__":
    main()



from openai import OpenAI
import json
import os

from src.tsbuddy import main as tsbuddy_main
from src.extracttar.extract_all import main as extract_all_main
from src.aosdl.aosdl import main as aosdl_main, lookup_ga_build, aosup
from src.logparser import main as logparser_main
from src.get_techsupport import main as get_techsupport_main
#from src.analyze.graph_hmon import main as graph_hmon_main

ENV_FILE = os.path.join(os.path.expanduser("~"), ".tsbuddy_secrets")

def load_env_file():
    """Load key-value pairs from .env into os.environ"""
    if os.path.exists(ENV_FILE):
        with open(ENV_FILE) as f:
            for line in f:
                if line.strip() and not line.startswith("#"):
                    key, sep, value = line.strip().partition("=")
                    if sep:  # Only set if '=' was found
                        os.environ.setdefault(key, value)

def append_to_env_file(key, value):
    """Append a new key=value to .env"""
    with open(ENV_FILE, "a") as f:
        f.write(f"{key}={value}\n")

# Load .env into environment
load_env_file()

# Prompt if API key not set
if "OPENAI_API_KEY" not in os.environ:
    api_key = input("Enter your OpenAI API key: ").strip()
    os.environ["OPENAI_API_KEY"] = api_key
    append_to_env_file("OPENAI_API_KEY", api_key)

# Use the key
# openai.api_key = os.environ["OPENAI_API_KEY"]
client = OpenAI(api_key=os.environ["OPENAI_API_KEY"])


# Define your function wrappers
def function_router(name, args=None):
    print(f"Calling function: {name}")
    if name == "lookup_ga_build":
        return lookup_ga_build()
    elif name == "get_techsupport_main":
        return get_techsupport_main()
    elif name == "extract_all_main":
        return extract_all_main()
    elif name == "tsbuddy_main":
        return tsbuddy_main()
    elif name == "logparser_main":
        return logparser_main()
    elif name == "aosup":
        return aosup()
    elif name == "aosdl_main":
        return aosdl_main()
    elif name == "graph_hmon_main":
        return graph_hmon_main()
    elif name == "change_directory":
        return change_directory()
    elif name == "upgrade_downgrade_choice":
        return upgrade_downgrade_choice()
    elif name == "print_help":
        return print_help()
    else:
        return "Unknown function."

# Register functions for OpenAI API
functions = [
    {
        "name": "lookup_ga_build",
        "description": "Get GA Build, Family, or Upgrade (aosga)",
        "parameters": {"type": "object", "properties": {}, "required": []}
    },
    {
        "name": "get_techsupport_main",
        "description": "Run tech support gatherer (ts-get)",
        "parameters": {"type": "object", "properties": {}, "required": []}
    },
    {
        "name": "extract_all_main",
        "description": "Run tech_support_complete.tar extractor (ts-extract)",
        "parameters": {"type": "object", "properties": {}, "required": []}
    },
    {
        "name": "tsbuddy_main",
        "description": "Run tech_support.log to CSV converter (ts-csv)",
        "parameters": {"type": "object", "properties": {}, "required": []}
    },
    {
        "name": "logparser_main",
        "description": "Run swlog parser to CSV & JSON (ts-log)",
        "parameters": {"type": "object", "properties": {}, "required": []}
    },
    {
        "name": "aosup",
        "description": "Run AOS Upgrader (aosup)",
        "parameters": {"type": "object", "properties": {}, "required": []}
    },
    {
        "name": "aosdl_main",
        "description": "Run AOS Downloader (aosdl)",
        "parameters": {"type": "object", "properties": {}, "required": []}
    },
    {
        "name": "graph_hmon_main",
        "description": "Run HMON Graph tool (ts-graph-hmon)",
        "parameters": {"type": "object", "properties": {}, "required": []}
    },
    {
        "name": "change_directory",
        "description": "Change current working directory",
        "parameters": {"type": "object", "properties": {}, "required": []}
    },
    {
        "name": "upgrade_downgrade_choice",
        "description": "Upgrade or downgrade tsbuddy",
        "parameters": {"type": "object", "properties": {}, "required": []}
    },
    {
        "name": "print_help",
        "description": "Show help info",
        "parameters": {"type": "object", "properties": {}, "required": []}
    }
]

# Main chatbot loop
def main():
    print("💬 Chatbot is ready. Type 'exit' to quit.")
    while True:
        user_input = input("You: ")
        if user_input.lower() in ("exit", "quit"):
            break

        response = client.chat.completions.create(
            model="gpt-5-nano",
            messages=[{"role": "user", "content": user_input}],
            functions=functions,
            function_call="auto"
        )
        message = response.choices[0].message
        if message.function_call:
            fn_name = message.function_call.name
            args = json.loads(message.function_call.arguments or "{}")
            result = function_router(fn_name, args)
            print(f"🤖 Result: {result}")
        else:
            print("🤖", message["content"])

if __name__ == "__main__":
    main()



import sqlite3, os
import pandas as pd

def CleanOutput(string):
#Remove unneeded characters
    string = string.replace("[", "")
    string = string.replace("]", "")
    string = string.replace(",", "")
    string = string.replace("(", "")
    string = string.replace(")", "")
    string = string.replace("'", "")
    return string

def main():
#Setup Pandas
    src_dir = os.path.dirname(os.path.abspath(__file__))
    console = pd.read_csv("ConsoleLogsParsed.csv",index_col=False)
    reboots = pd.read_csv(src_dir+'/loglist-reboots.csv',index_col=False)
    #replaces NaN with an empty string. Otherwise, we'll get "argument of type 'float' is not iterable"
    reboots = reboots.fillna('')
    console = console.fillna('')
#Initialize Database
    with sqlite3.connect(':memory:') as conn:
        console.to_sql("TempLogs", conn, if_exists='append', index=False)
        reboots.to_sql("Checks", conn, if_exists='append', index=False)
#Make new table with Primary Key
        cursor = conn.cursor()
        cursor.execute("create table Logs(id integer primary key autoincrement, Year integer, Month text, Day integer, Time integer, SwitchName Text, AppID Text, Subapp Text, Priority text, LogMessage text)")
        cursor.execute("insert into Logs (Year, Month, Day, Time, SwitchName, AppID, Subapp, Priority, LogMessage) select Year, Month, Day, Time, SwitchName, AppID, Subapp, Priority, LogMessage from TempLogs")
        cursor.execute("drop table TempLogs")
        conn.commit()
#Compare
    #Get Number of rows in Check
        cursor.execute("select count(*) from Checks")
        CheckCount = cursor.fetchall()
    #Remove extra characters
        CheckCount = str(CheckCount)
        CheckCount = CleanOutput(CheckCount)
        CheckCount = int(CheckCount)
        #print(CheckCount)
        cursor.execute("Select * from Checks")
        output = cursor.fetchall()
        
    #Clean and compare. Add LogMeaning to matched logs.
        cursor.execute("alter table Logs add column LogMeaning")
        cursor.execute("alter table Logs add column LogMatch")
        cursor.execute("update Logs set LogMatch = 'false'")
        cursor.execute("alter table Logs add column PreviousTime")
        cursor.execute("alter table Logs add column TimeDiff")
        for line in output:
            line = str(line)
            parts = line.split(', ')
            Message = parts[0]
            Meaning = parts[1]
            Message = CleanOutput(Message)
            Meaning = CleanOutput(Meaning)
            cursor.execute("update Logs set LogMeaning = '"+Meaning+"' where LogMessage like '%"+Message+"%'")
            cursor.execute("update Logs set LogMatch = 'true' where LogMessage like '%"+Message+"%'")
#Set PreviousTime
        cursor.execute("select id from Logs where LogMatch = 'true'")
        output = cursor.fetchall()
        #print(output)
        matchIDs = []
        RebootIDs = []
        FirstRebootTime = ""
        for line in output:
            line = str(line)
            matchIDs.append(CleanOutput(line))
        counter = 0
        #print(len(matchIDs))
        for id in matchIDs:
            if counter == 0:
                cursor.execute("select Year,Month,Day,Time from Logs where id = "+str(id))
                output = cursor.fetchall()
                output = str(output)
                FirstRebootTime = CleanOutput(output)
                cursor.execute("update Logs set PreviousTime = 'First Log' where id = "+matchIDs[counter-1])
                counter += 1
                continue
            cursor.execute("select time from Logs where ID = "+id)
            output = cursor.fetchall()
            output = str(output)
            Rowtime = CleanOutput(output)
            #print(Rowtime)
            cursor.execute("update Logs set PreviousTime = '"+Rowtime+"' where id = "+matchIDs[counter-1])
            #print(Rowid)
            #print(Rowtime)
#Set TimeDiff. Convert provided timestamp to seconds, compare, 
# mark any logs with a difference greater than 5 minutes and convert back to timestamp
            RowtimeParts = Rowtime.split('.')
            RowtimeParts = RowtimeParts[0].split(':')
            Rowtime_hr = int(RowtimeParts[0])
            Rowtime_mn = int(RowtimeParts[1])
            Rowtime_sc = int(RowtimeParts[2])
            Rowtime_totalseconds = ((Rowtime_hr*3600)+(Rowtime_mn*60)+(Rowtime_sc))
            #PreviousTime
            cursor.execute("select time from Logs where id = "+matchIDs[counter-1])
            PreviousTime = cursor.fetchall()
            PreviousTime = CleanOutput(str(PreviousTime))
            PreviousTime = PreviousTime.split('.')
            PreviousTime = PreviousTime[0].split(':')
            PreviousTime_hr = int(PreviousTime[0])
            PreviousTime_mn = int(PreviousTime[1])
            PreviousTime_sc = int(PreviousTime[2])
            PreviousTime_totalseconds = ((PreviousTime_hr*3600)+(PreviousTime_mn*60)+(PreviousTime_sc))
            #print("Rowtime: "+str(Rowtime_totalseconds))
            #print("PreviousTime: "+str(PreviousTime_totalseconds))
            Diff = abs(Rowtime_totalseconds-PreviousTime_totalseconds)
            if Diff >= 3600:
                RebootIDs.append(id)
            DiffTime_hr,Remainder = divmod(Diff, 3600)
            DiffTime_mn,DiffTime_sc = divmod(Remainder, 60)
            DiffTime_hr = str(DiffTime_hr)
            DiffTime_mn = str(DiffTime_mn)
            DiffTime_sc = str(DiffTime_sc)
            if len(DiffTime_hr) == 1:
                DiffTime_hr = "0"+DiffTime_hr
            if len(DiffTime_mn) == 1:
                DiffTime_mn = "0"+DiffTime_mn
            if len(DiffTime_sc) == 1:
                DiffTime_sc = "0"+DiffTime_sc
            DiffTime = (DiffTime_hr+":"+DiffTime_mn+":"+DiffTime_sc)
            #print(Diff)
            #print(DiffTime_hr)
            #print(DiffTime_mn)
            #print(DiffTime_sc)
            #print(DiffTime)
            cursor.execute("update Logs set TimeDiff = '"+str(DiffTime)+"' where id = "+matchIDs[counter-1])
            counter += 1

#Select columns and export        
        Output = pd.read_sql("SELECT Year,Month,Day,Time,TimeDiff,SwitchName,AppID,Subapp,Priority,LogMessage,LogMeaning from Logs order by LogMatch desc", conn)
        Output.to_excel("tsbuddy-ConsoleLogAnalysis-Reboots.xlsx")

#List reboot count and times
        RebootCount = len(RebootIDs)
        RebootTimes = []
        for id in RebootIDs:
            cursor.execute("select Year,Month,Day,Time from Logs where id = "+str(id))
            output = cursor.fetchall()
            output = str(output)
            RebootTime = CleanOutput(output)
            RebootTimes.append(RebootTime)
        if RebootCount == 0 and len(matchIDs) != 0:
            print("This switch rebooted once around: "+FirstRebootTime)
        if RebootCount == 0 and len(matchIDs) == 0:
            print("There are no reboots in these switch logs.")
        if RebootCount > 0:
            print("This switch rebooted "+str(RebootCount)+" times around the following timestamps:")
            for time in RebootTimes:
                print(time)

main()